@@ -6,8 +6,9 @@ Usage:
   runway (plan|taxi)
   runway (deploy|takeoff)
   runway (destroy|dismantle)
+  runway init
   runway gitclean
-  runway gen-sample (cfn|sls|stacker|tf)
+  runway gen-sample (cfn|sls|stacker|tf|cdk)
   runway whichenv
   runway -h | --help
   runway --version
