@@ -3,9 +3,11 @@ from .deploy import Deploy # noqa
 from .destroy import Destroy # noqa
 from .dismantle import Dismantle # noqa
 from .gitclean import GitClean # noqa
+from .init import Init # noqa
 from .plan import Plan  # noqa
 from .preflight import Preflight  # noqa
 from .test import Test  # noqa
 from .takeoff import Takeoff  # noqa
 from .taxi import Taxi  # noqa
 from .gen_sample import GenSample  # noqa
+from .whichenv import WhichEnv  # noqa
