@@ -9,6 +9,7 @@ import os
 import shutil
 import sys
 
+import cfn_flip
 import yaml
 
 # from stacker.util import parse_cloudformation_template
@@ -257,12 +258,27 @@ class Base(object):
         LOGGER.info("Sample Serverless module created at %s",
                     module_dir)
 
+    def generate_sample_cdk_module(self, module_dir=None):
+        """Generate skeleton CDK sample module."""
+        if module_dir is None:
+            module_dir = os.path.join(self.env_root, 'sampleapp.cdk')
+        self.generate_sample_module(module_dir)
+        for i in ['cdk.json', 'index.ts', 'package.json', 'tsconfig.json']:
+            shutil.copyfile(
+                os.path.join(os.path.dirname(os.path.dirname(__file__)),
+                             'templates',
+                             'cdk',
+                             i),
+                os.path.join(module_dir, i),
+            )
+        LOGGER.info("Sample CDK module created at %s", module_dir)
+
     def generate_sample_cfn_module(self, module_dir=None):
         """Generate skeleton CloudFormation sample module."""
         if module_dir is None:
             module_dir = os.path.join(self.env_root, 'sampleapp.cfn')
         self.generate_sample_module(module_dir)
-        for i in ['01-sampleapp.yaml', 'dev-us-east-1.env']:
+        for i in ['stacks.yaml', 'dev-us-east-1.env']:
             shutil.copyfile(
                 os.path.join(os.path.dirname(os.path.dirname(__file__)),
                              'templates',
@@ -271,14 +287,22 @@ class Base(object):
                 os.path.join(module_dir, i)
             )
         os.mkdir(os.path.join(module_dir, 'templates'))
-        shutil.copyfile(
-            os.path.join(os.path.dirname(os.path.dirname(__file__)),
-                         'templates',
-                         'cfn',
-                         'templates',
-                         'bucket.json'),
-            os.path.join(module_dir, 'templates', 'bucket.json')
-        )
+        with open(os.path.join(module_dir,
+                               'templates',
+                               'tf_state.yml'), 'w') as stream:
+            stream.write(
+                cfn_flip.flip(
+                    check_output(
+                        [sys.executable,
+                         os.path.join(os.path.dirname(os.path.dirname(__file__)),  # noqa
+                                      'templates',
+                                      'stacker',
+                                      'tfstate_blueprints',
+                                      'tf_state.py')]
+                    )
+
+                )
+            )
         LOGGER.info("Sample CloudFormation module created at %s",
                     module_dir)
 
