@@ -59,7 +59,9 @@ def determine_module_class(path, module_options):
         if os.path.basename(path).endswith('.sls'):
             module_options['class_path'] = 'runway.module.serverless.Serverless'  # noqa
         elif os.path.basename(path).endswith('.tf'):
-            module_options['class_path'] = 'runway.module.terraform.Terraform'  # noqa
+            module_options['class_path'] = 'runway.module.terraform.Terraform'
+        elif os.path.basename(path).endswith('.cdk'):
+            module_options['class_path'] = 'runway.module.cdk.CloudDevelopmentKit'  # noqa
         elif os.path.basename(path).endswith('.cfn'):
             module_options['class_path'] = 'runway.module.cloudformation.CloudFormation'  # noqa
         # Fallback to autodetection
@@ -67,7 +69,10 @@ def determine_module_class(path, module_options):
                                          'serverless.yml')):
             module_options['class_path'] = 'runway.module.serverless.Serverless'  # noqa
         elif glob.glob(os.path.join(path, '*.tf')):
-            module_options['class_path'] = 'runway.module.terraform.Terraform'  # noqa
+            module_options['class_path'] = 'runway.module.terraform.Terraform'
+        elif os.path.isfile(os.path.join(path, 'cdk.json')) and (
+                os.path.isfile(os.path.join(path, 'package.json'))):
+            module_options['class_path'] = 'runway.module.cdk.CloudDevelopmentKit'  # noqa
         elif glob.glob(os.path.join(path, '*.env')):
             module_options['class_path'] = 'runway.module.cloudformation.CloudFormation'  # noqa
     if not module_options.get('class_path'):
@@ -77,6 +82,24 @@ def determine_module_class(path, module_options):
     return load_object_from_string(module_options['class_path'])
 
 
+def get_deployment_env_vars(env_name, env_var_config=None, env_root=None):
+    """Return applicable environment variables."""
+    if env_var_config is None:
+        env_var_config = {}
+    if env_var_config.get('*'):
+        env_vars = env_var_config['*'].copy()
+    else:
+        env_vars = {}
+    if env_var_config.get(env_name):
+        for (key, val) in env_var_config[env_name].items():
+            # Lists are presumed to be path components and will be turned back
+            # to strings
+            if isinstance(val, list):
+                env_var_config[env_name][key] = os.path.join(env_root, os.path.join(*val)) if (env_root and not os.path.isabs(os.path.join(*val))) else os.path.join(*val)  # noqa pylint: disable=line-too-long
+        env_vars = merge_dicts(env_vars, env_var_config[env_name])
+    return env_vars
+
+
 def get_env_from_branch(branch_name):
     """Determine environment name from git branch name."""
     if branch_name.startswith('ENV-'):
@@ -124,6 +147,13 @@ def get_env(path, ignore_git_branch=False):
     return get_env_from_directory(os.path.basename(path))
 
 
+def path_is_current_dir(path):
+    """Determine if defined path is reference to current directory."""
+    if path in ['.', '.' + os.sep]:
+        return True
+    return False
+
+
 def load_module_opts_from_file(path, module_options):
     """Update module_options with any options defined in module path."""
     module_options_file = os.path.join(path,
@@ -145,6 +175,7 @@ def post_deploy_assume_role(assume_role_config, context):
 def pre_deploy_assume_role(assume_role_config, context):
     """Assume role (prior to deployment)."""
     if isinstance(assume_role_config, dict):
+        assume_role_arn = ''
         if assume_role_config.get('post_deploy_env_revert'):
             context.save_existing_iam_env_vars()
         if assume_role_config.get('arn'):
@@ -162,16 +193,17 @@ def pre_deploy_assume_role(assume_role_config, context):
                         'environment %s...',
                         context.env_name)
 
-        context.env_vars = merge_dicts(
-            context.env_vars,
-            assume_role(
-                role_arn=assume_role_arn,
-                session_name=assume_role_config.get('session_name', None),
-                duration_seconds=assume_role_duration,
-                region=context.env_region,
-                env_vars=context.env_vars
+        if assume_role_arn:
+            context.env_vars = merge_dicts(
+                context.env_vars,
+                assume_role(
+                    role_arn=assume_role_arn,
+                    session_name=assume_role_config.get('session_name', None),
+                    duration_seconds=assume_role_duration,
+                    region=context.env_region,
+                    env_vars=context.env_vars
+                )
             )
-        )
     else:
         context.env_vars = merge_dicts(
             context.env_vars,
@@ -269,7 +301,7 @@ class Env(Base):
             shutil.rmtree(os.path.join(self.env_root, directory))
         return True
 
-    def run(self, deployments=None, command='plan'):  # noqa pylint: disable=too-many-branches
+    def run(self, deployments=None, command='plan'):  # noqa pylint: disable=too-many-branches,too-many-statements
         """Execute apps/code command."""
         if deployments is None:
             deployments = self.runway_config['deployments']
@@ -306,6 +338,13 @@ class Env(Base):
                 )
         for deployment in deployments_to_run:
             if deployment.get('regions'):
+                if deployment.get('env_vars'):
+                    context.env_vars = merge_dicts(
+                        context.env_vars,
+                        get_deployment_env_vars(context.env_name,
+                                                deployment['env_vars'],
+                                                self.env_root)
+                    )
                 for region in deployment['regions']:
                     context.env_region = region
                     context.env_vars = merge_dicts(
@@ -313,26 +352,34 @@ class Env(Base):
                         {'AWS_DEFAULT_REGION': context.env_region,
                          'AWS_REGION': context.env_region}
                     )
-                    module_opts = {}
-                    if deployment.get('environments'):
-                        module_opts['environments'] = deployment['environments'].copy()  # noqa
                     if deployment.get('assume-role'):
                         pre_deploy_assume_role(deployment['assume-role'],
                                                context)
                     if deployment.get('account-id') or (
                             deployment.get('account-alias')):
                         validate_account_credentials(deployment, context)
-                    for module in deployment.get('modules', []):
+                    modules = deployment.get('modules', [])
+                    if deployment.get('current_dir'):
+                        modules.append('.' + os.sep)
+                    for module in modules:
+                        module_opts = {}
+                        if deployment.get('environments'):
+                            module_opts['environments'] = deployment['environments'].copy()  # noqa
                         if isinstance(module, six.string_types):
-                            module_root = os.path.join(self.env_root, module)
+                            module = {'path': module}
+                        if path_is_current_dir(module['path']):
+                            module_root = self.env_root
                         else:
                             module_root = os.path.join(self.env_root,
                                                        module['path'])
-                            module_opts = merge_dicts(module_opts, module)
+                        module_opts = merge_dicts(module_opts, module)
                         module_opts = load_module_opts_from_file(module_root,
                                                                  module_opts)
                         if deployment.get('skip-npm-ci'):
                             module_opts['skip_npm_ci'] = True
+                        LOGGER.info("Processing module %s...\n",
+                                    module['path'])
+                        LOGGER.debug("Module options are: %s", module_opts)
                         with change_dir(module_root):
                             getattr(
                                 determine_module_class(module_root, module_opts)(  # noqa
@@ -341,16 +388,11 @@ class Env(Base):
                                     options=module_opts
                                 ),
                                 command)()
-                    if deployment.get('current_dir', False):
-                        getattr(
-                            determine_module_class(module_root, module_opts)(
-                                context=context,
-                                path=context.env_root,
-                                options=module_opts
-                            ),
-                            command)()
                 if deployment.get('assume-role'):
                     post_deploy_assume_role(deployment['assume-role'], context)
+            else:
+                LOGGER.error('No region configured for any deployment')
+                sys.exit(1)
 
     def plan(self, deployments=None):
         """Plan apps/code deployment."""
