@@ -1,3 +1,6 @@
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
 import logging
 
 from .base import BaseAction, plan, build_walker
@@ -30,8 +33,7 @@ logger = logging.getLogger(__name__)
 
 def build_stack_tags(stack):
     """Builds a common set of tags to attach to a stack"""
-    return [
-        {'Key': t[0], 'Value': t[1]} for t in stack.tags.items()]
+    return [{'Key': t[0], 'Value': t[1]} for t in stack.tags.items()]
 
 
 def should_update(stack):
@@ -72,6 +74,20 @@ def should_submit(stack):
     return False
 
 
+def should_ensure_cfn_bucket(outline, dump):
+    """Test whether access to the cloudformation template bucket is required
+
+    Args:
+        outline (bool): The outline action.
+        dump (bool): The dump action.
+
+    Returns:
+        bool: If access to CF bucket is needed, return True.
+
+    """
+    return not outline and not dump
+
+
 def _resolve_parameters(parameters, blueprint):
     """Resolves CloudFormation Parameters for a given blueprint.
 
@@ -111,14 +127,26 @@ def _resolve_parameters(parameters, blueprint):
     return params
 
 
-def _handle_missing_parameters(params, required_params, existing_stack=None):
+class UsePreviousParameterValue(object):
+    """ A simple class used to indicate a Parameter should use it's existng
+    value.
+    """
+    pass
+
+
+def _handle_missing_parameters(parameter_values, all_params, required_params,
+                               existing_stack=None):
     """Handles any missing parameters.
 
     If an existing_stack is provided, look up missing parameters there.
 
     Args:
-        params (dict): key/value dictionary of stack definition parameters
-        required_params (list): A list of required parameter names.
+        parameter_values (dict): key/value dictionary of stack definition
+            parameters
+        all_params (list): A list of all the parameters used by the
+            template/blueprint.
+        required_params (list): A list of all the parameters required by the
+            template/blueprint.
         existing_stack (dict): A dict representation of the stack. If
             provided, will be searched for any missing parameters.
 
@@ -131,21 +159,24 @@ def _handle_missing_parameters(params, required_params, existing_stack=None):
             still missing.
 
     """
-    missing_params = list(set(required_params) - set(params.keys()))
+    missing_params = list(set(all_params) - set(parameter_values.keys()))
     if existing_stack and 'Parameters' in existing_stack:
-        stack_params = {p['ParameterKey']: p['ParameterValue'] for p in
-                        existing_stack['Parameters']}
+        stack_parameters = [
+            p["ParameterKey"] for p in existing_stack["Parameters"]
+        ]
         for p in missing_params:
-            if p in stack_params:
-                value = stack_params[p]
-                logger.debug("Using parameter %s from existing stack: %s",
-                             p, value)
-                params[p] = value
-    final_missing = list(set(required_params) - set(params.keys()))
+            if p in stack_parameters:
+                logger.debug(
+                    "Using previous value for parameter %s from existing "
+                    "stack",
+                    p
+                )
+                parameter_values[p] = UsePreviousParameterValue
+    final_missing = list(set(required_params) - set(parameter_values.keys()))
     if final_missing:
         raise MissingParameterException(final_missing)
 
-    return params.items()
+    return list(parameter_values.items())
 
 
 def handle_hooks(stage, hooks, provider, context, dump, outline):
@@ -198,13 +229,24 @@ class Action(BaseAction):
 
         """
         resolved = _resolve_parameters(stack.parameter_values, stack.blueprint)
-        required_parameters = stack.required_parameter_definitions.keys()
-        parameters = _handle_missing_parameters(resolved, required_parameters,
+        required_parameters = list(stack.required_parameter_definitions)
+        all_parameters = list(stack.all_parameter_definitions)
+        parameters = _handle_missing_parameters(resolved, all_parameters,
+                                                required_parameters,
                                                 provider_stack)
-        return [
-            {'ParameterKey': p[0],
-             'ParameterValue': str(p[1])} for p in parameters
-        ]
+
+        param_list = []
+
+        for key, value in parameters:
+            param_dict = {"ParameterKey": key}
+            if value is UsePreviousParameterValue:
+                param_dict["UsePreviousValue"] = True
+            else:
+                param_dict["ParameterValue"] = str(value)
+
+            param_list.append(param_dict)
+
+        return param_list
 
     def _launch_stack(self, stack, **kwargs):
         """Handles the creating or updating of a stack in CloudFormation.
@@ -344,14 +386,14 @@ class Action(BaseAction):
     def _generate_plan(self, tail=False):
         return plan(
             description="Create/Update stacks",
-            action=self._launch_stack,
+            stack_action=self._launch_stack,
             tail=self._tail_stack if tail else None,
-            stacks=self.context.get_stacks(),
-            targets=self.context.stack_names)
+            context=self.context)
 
     def pre_run(self, outline=False, dump=False, *args, **kwargs):
         """Any steps that need to be taken prior to running the action."""
-        self.ensure_cfn_bucket()
+        if should_ensure_cfn_bucket(outline, dump):
+            self.ensure_cfn_bucket()
         hooks = self.context.config.pre_build
         handle_hooks(
             "pre_build",
@@ -370,6 +412,8 @@ class Action(BaseAction):
 
         """
         plan = self._generate_plan(tail=tail)
+        if not plan.keys():
+            logger.warn('WARNING: No stacks detected (error in config?)')
         if not outline and not dump:
             plan.outline(logging.DEBUG)
             logger.debug("Launching stacks: %s", ", ".join(plan.keys()))
