@@ -1,3 +1,6 @@
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
 import logging
 
 from .build import Build
@@ -27,13 +30,13 @@ class Stacker(BaseCommand):
         else:
             logger.info("Using default AWS provider mode")
 
+        session_cache.default_profile = options.profile
+
         config = load_config(
             options.config.read(),
             environment=options.environment,
             validate=True)
 
-        session_cache.default_profile = options.profile
-
         options.provider_builder = default.ProviderBuilder(
             region=options.region,
             interactive=options.interactive,
