@@ -1,3 +1,7 @@
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
+from builtins import object
 import copy
 
 from . import util
@@ -40,7 +44,7 @@ def _gather_variables(stack_def):
             variables.
     """
     variable_values = copy.deepcopy(stack_def.variables or {})
-    return [Variable(k, v) for k, v in variable_values.iteritems()]
+    return [Variable(k, v) for k, v in variable_values.items()]
 
 
 class Stack(object):
@@ -61,6 +65,7 @@ class Stack(object):
 
     def __init__(self, definition, context, variables=None, mappings=None,
                  locked=False, force=False, enabled=True, protected=False):
+        self.logging = True
         self.name = definition.name
         self.fqn = context.get_fqn(definition.stack_name or self.name)
         self.region = definition.region
@@ -72,14 +77,24 @@ class Stack(object):
         self.force = force
         self.enabled = enabled
         self.protected = protected
-        self.context = copy.deepcopy(context)
+        self.context = context
         self.outputs = None
 
     def __repr__(self):
         return self.fqn
 
+    @property
+    def required_by(self):
+        return self.definition.required_by or []
+
     @property
     def requires(self):
+        # By definition, a locked stack has no dependencies, because we won't
+        # be performing an update operation on the stack. This means, resolving
+        # outputs from dependencies is unnecessary.
+        if self.locked and not self.force:
+            return []
+
         requires = set(self.definition.requires or [])
 
         # Add any dependencies based on output lookups
@@ -90,7 +105,7 @@ class Stack(object):
                     try:
                         d = deconstruct(lookup.input)
                     except ValueError as e:
-                        raise FailedVariableLookup(self.name, e)
+                        raise FailedVariableLookup(self.name, lookup, e)
 
                     if d.stack_name == self.name:
                         message = (
@@ -166,6 +181,11 @@ class Stack(object):
         """
         return self.blueprint.get_parameter_values()
 
+    @property
+    def all_parameter_definitions(self):
+        """Return a list of all parameters in the blueprint/template."""
+        return self.blueprint.get_parameter_definitions()
+
     @property
     def required_parameter_definitions(self):
         """Return all the required CloudFormation Parameters for the stack."""
