@@ -1,3 +1,8 @@
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
+from builtins import str
+from builtins import object
 import copy
 import uuid
 import importlib
@@ -225,7 +230,7 @@ def merge_map(a, b):
     if not isinstance(a, dict) or not isinstance(b, dict):
         return b
 
-    for key in b.keys():
+    for key in b:
         a[key] = merge_map(a[key], b[key]) if key in a else b[key]
     return a
 
@@ -275,12 +280,12 @@ def yaml_to_ordered_dict(stream, loader=yaml.SafeLoader):
                     None, None,
                     "expected a mapping node, but found %s" % node.id,
                     node.start_mark)
-            mapping = {}
+            mapping = OrderedDict()
             for key_node, value_node in node.value:
                 key = self.construct_object(key_node, deep=deep)
                 try:
                     hash(key)
-                except TypeError, exc:
+                except TypeError as exc:
                     raise ConstructorError(
                         "while constructing a mapping", node.start_mark,
                         "found unhashable key (%s)" % exc, key_node.start_mark
@@ -302,8 +307,17 @@ def yaml_to_ordered_dict(stream, loader=yaml.SafeLoader):
             """Override parent method to use OrderedDict."""
             if isinstance(node, MappingNode):
                 self.flatten_mapping(node)
-            return OrderedDict(self._validate_mapping(node, deep=deep))
+            return self._validate_mapping(node, deep=deep)
 
+        def construct_yaml_map(self, node):
+            data = OrderedDict()
+            yield data
+            value = self.construct_mapping(node)
+            data.update(value)
+
+    OrderedUniqueLoader.add_constructor(
+        u'tag:yaml.org,2002:map', OrderedUniqueLoader.construct_yaml_map,
+    )
     return yaml.load(stream, OrderedUniqueLoader)
 
 
@@ -354,7 +368,8 @@ def handle_hooks(stage, hooks, provider, context):
         data_key = hook.data_key
         required = hook.required
         kwargs = hook.args or {}
-        if not hook.enabled:
+        enabled = hook.enabled
+        if not enabled:
             logger.debug("hook with method %s is disabled, skipping",
                          hook.path)
             continue
@@ -613,6 +628,9 @@ class SourceProcessor(object):
 
     def get_package_sources(self):
         """Make remote python packages available for local use."""
+        # Checkout local modules
+        for config in self.sources.get('local', []):
+            self.fetch_local_package(config=config)
         # Checkout S3 repositories specified in config
         for config in self.sources.get('s3', []):
             self.fetch_s3_package(config=config)
@@ -620,6 +638,18 @@ class SourceProcessor(object):
         for config in self.sources.get('git', []):
             self.fetch_git_package(config=config)
 
+    def fetch_local_package(self, config):
+        """Make a local path available to current stacker config.
+
+        Args:
+            config (dict): 'local' path config dictionary
+
+        """
+        # Update sys.path & merge in remote configs (if necessary)
+        self.update_paths_and_config(config=config,
+                                     pkg_dir_name=config['source'],
+                                     pkg_cache_dir=os.getcwd())
+
     def fetch_s3_package(self, config):
         """Make a remote S3 archive available for local use.
 
@@ -631,7 +661,7 @@ class SourceProcessor(object):
                          '.tar': TarExtractor,
                          '.zip': ZipExtractor}
         extractor = None
-        for suffix, klass in extractor_map.iteritems():
+        for suffix, klass in extractor_map.items():
             if config['key'].endswith(suffix):
                 extractor = klass()
                 logger.debug("Using extractor %s for S3 object \"%s\" in "
@@ -758,21 +788,25 @@ class SourceProcessor(object):
         self.update_paths_and_config(config=config,
                                      pkg_dir_name=dir_name)
 
-    def update_paths_and_config(self, config, pkg_dir_name):
+    def update_paths_and_config(self, config, pkg_dir_name,
+                                pkg_cache_dir=None):
         """Handle remote source defined sys.paths & configs.
 
         Args:
             config (dict): git config dictionary
             pkg_dir_name (string): directory name of the stacker archive
+            pkg_cache_dir (string): fully qualified path to stacker cache
+                                    cache directory
 
         """
-        cached_dir_path = os.path.join(self.package_cache_dir, pkg_dir_name)
+        if pkg_cache_dir is None:
+            pkg_cache_dir = self.package_cache_dir
+        cached_dir_path = os.path.join(pkg_cache_dir, pkg_dir_name)
 
         # Add the appropriate directory (or directories) to sys.path
         if config.get('paths'):
             for path in config['paths']:
-                path_to_append = os.path.join(self.package_cache_dir,
-                                              pkg_dir_name,
+                path_to_append = os.path.join(cached_dir_path,
                                               path)
                 logger.debug("Appending \"%s\" to python sys.path",
                              path_to_append)
@@ -803,8 +837,8 @@ class SourceProcessor(object):
                                                    'ls-remote',
                                                    uri,
                                                    ref])
-        if "\t" in lsremote_output:
-            commit_id = lsremote_output.split("\t")[0]
+        if b"\t" in lsremote_output:
+            commit_id = lsremote_output.split(b"\t")[0]
             logger.debug("Matching commit id found: %s", commit_id)
             return commit_id
         else:
@@ -862,6 +896,8 @@ class SourceProcessor(object):
                 config['uri'],
                 self.determine_git_ls_remote_ref(config)
             )
+        if sys.version_info[0] > 2 and isinstance(ref, bytes):
+            return ref.decode()
         return ref
 
     def sanitize_uri_path(self, uri):
