@@ -2,45 +2,22 @@
 
 import logging
 import os
-from subprocess import check_call
 import tempfile
 import zipfile
 
 from boto3.s3.transfer import S3Transfer
 import boto3
-import six
 
 from botocore.exceptions import ClientError
 from stacker.lookups.handlers.rxref import handler as rxref_handler
 from stacker.session_cache import get_session
 
 from .util import get_hash_of_files
-from ...util import change_dir
+from ...util import change_dir, run_commands
 
 LOGGER = logging.getLogger(__name__)
 
 
-def run_commands(commands, directory):
-    # type: (List[Union[str, List[str], Dict[str, Union[str, List[str]]]]],
-    #        str)
-    # -> None
-    """Run list of commands."""
-    for step in commands:
-        if isinstance(step, (list, six.string_types)):
-            execution_dir = directory
-            raw_command = step
-        elif step.get('command'):  # dictionary
-            execution_dir = os.path.join(directory,
-                                         step.get('cwd')) if step.get('cwd') else directory  # noqa pylint: disable=line-too-long
-            raw_command = step['command']
-        else:
-            raise AttributeError("Invalid command step: %s" % step)
-        command_list = raw_command.split(' ') if isinstance(raw_command, six.string_types) else raw_command  # noqa pylint: disable=line-too-long
-
-        with change_dir(execution_dir):
-            check_call(command_list, env=os.environ)
-
-
 def does_s3_object_exist(bucket_name, key, session=None):
     """Determine if object exists on s3."""
     if session:
@@ -104,9 +81,9 @@ def build(context, provider, **kwargs):  # pylint: disable=unused-argument
     """Build static site."""
     session = get_session(provider.region)
     options = kwargs.get('options', {})
-    artifact_key_prefix = "%s-%s-" % (options['namespace'], options['name'])
-    default_param_name = "%shash" % artifact_key_prefix
     context_dict = {}
+    context_dict['artifact_key_prefix'] = "%s-%s-" % (options['namespace'], options['name'])  # noqa
+    default_param_name = "%shash" % context_dict['artifact_key_prefix']
 
     if options.get('build_output'):
         build_output = os.path.join(
@@ -116,7 +93,7 @@ def build(context, provider, **kwargs):  # pylint: disable=unused-argument
     else:
         build_output = options['path']
 
-    artifact_bucket_name = rxref_handler(
+    context_dict['artifact_bucket_name'] = rxref_handler(
         kwargs.get('artifact_bucket_rxref_lookup'),
         provider=provider,
         context=context
@@ -147,6 +124,14 @@ def build(context, provider, **kwargs):  # pylint: disable=unused-argument
         context_dict['hash_tracking_disabled'] = True
         old_parameter_value = None
 
+    context_dict['current_archive_filename'] = (
+        context_dict['artifact_key_prefix'] + context_dict['hash'] + '.zip'
+    )
+    if old_parameter_value:
+        context_dict['old_archive_filename'] = (
+            context_dict['artifact_key_prefix'] + old_parameter_value + '.zip'
+        )
+
     if old_parameter_value == context_dict['hash']:
         LOGGER.info("staticsite: skipping build; app hash %s already deployed "
                     "in this environment",
@@ -154,17 +139,19 @@ def build(context, provider, **kwargs):  # pylint: disable=unused-argument
         context_dict['deploy_is_current'] = True
         return context_dict
 
-    archive_key = artifact_key_prefix + context_dict['hash'] + '.zip'
-    if does_s3_object_exist(artifact_bucket_name, archive_key, session):
+    if does_s3_object_exist(context_dict['artifact_bucket_name'],
+                            context_dict['current_archive_filename'],
+                            session):
         context_dict['app_directory'] = download_and_extract_to_mkdtemp(
-            artifact_bucket_name, archive_key, session
+            context_dict['artifact_bucket_name'],
+            context_dict['current_archive_filename'], session
         )
     else:
         if options.get('build_steps'):
             LOGGER.info('staticsite: executing build commands')
             run_commands(options['build_steps'], options['path'])
-        zip_and_upload(build_output, artifact_bucket_name, archive_key,
-                       session)
+        zip_and_upload(build_output, context_dict['artifact_bucket_name'],
+                       context_dict['current_archive_filename'], session)
         context_dict['app_directory'] = build_output
 
     context_dict['deploy_is_current'] = False
