@@ -2,6 +2,7 @@
 
 import logging
 import os
+import platform
 import re
 import sys
 
@@ -57,6 +58,10 @@ def make_stacker_cmd_string(args, lib_path):
     * Overriding sys.argv
     * Adding embedded runway lib directory to sys.path
     """
+    if platform.system().lower() == 'windows':
+        # Because this will be run via subprocess, the backslashes on Windows
+        # will cause command errors
+        lib_path = lib_path.replace('\\', '/')
     return ("import sys;"
             "sys.argv = ['stacker'] + {args};"
             "sys.path.insert(1, '{lib_path}');"
@@ -133,9 +138,10 @@ class CloudFormation(RunwayModule):
                         ensure_stacker_compat_config(
                             os.path.join(self.path, name)
                         )
-                        LOGGER.info("Running stacker %s on %s",
+                        LOGGER.info("Running stacker %s on %s in region %s",
                                     command,
-                                    name)
+                                    name,
+                                    self.context.env_region)
                         stacker_cmd_str = make_stacker_cmd_string(
                             stacker_cmd + [name],
                             get_embedded_lib_path()
